#!/bin/sh

if [ $# -lt 3 -o "$1" = "-h" -o "$1" = "--help" ]; then
	echo "Usage: $0 <hatari> <program> <cpulevel> ..."
	exit 1;
fi

hatari="$1"
shift
if [ ! -x "$hatari" ]; then
	echo "First parameter must point to valid hatari executable."
	exit 1;
fi;

program="$1"
shift
if [ ! -f "$program" ]; then
	echo "Second parameter must point to valid gemdos executable."
	exit 1;
fi;

cpulevel="$1"
shift
fpu="--fpu none"
case "$cpulevel" in
0|1|2) ;;
3) fpu="--fpu 68882" ;;
4|6) fpu="--fpu internal" ;;
*)
	echo "Third parameter must be cpulevel in range 0..4, or 6."
	exit 1
	;;
esac

basedir=$(dirname $0)
testdir=$(mktemp -d)

remove_temp() {
  if test "$program" != test-inf.ttp; then rm -rf "$testdir"; fi
}
trap remove_temp EXIT

export SDL_VIDEODRIVER=dummy
export SDL_AUDIODRIVER=dummy

RUNTEST_ARGS="$hatari --log-level debug --sound off --cpuclock 32 \
	--tos none -d $testdir --run-vbls 500 --bios-intercept on --cpulevel $cpulevel $fpu --fpu-softfloat 1 $@ $basedir/$program"
	
echo HOME="$testdir" $RUNTEST_ARGS > "$testdir/out.txt"
HOME="$testdir" $RUNTEST_ARGS 2>> "$testdir/out.txt"
exitstat=$?
if [ $exitstat -ne 0 ]; then
	echo "Running hatari failed. Status=${exitstat}."
	cat "$testdir/out.txt"
	exit 1
fi

exit 0
