/*
 * Test:USES_FPU
 */
#include "testdriver.h"

#ifndef __HAVE_68881__
 #error "must be compiled with FPU support"
#endif

int numtests;

union ld_union pi = { { 0x40000000L, 0xc90fdaa2L, 0x2168C235L } };

int main(void)
{
	int status;
	union ld_union fp0;
	union ld_union fp1;
	char fail;
	union {
		char c[12];
		short w[6];
		long l[3];
	} data;
	long d3;
	
	status = 0;
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x %[pi],%[fp0]\n"
		"\tfmovecr.x #0x00,%[fp1]\n"
		"\tfcmp.x %[fp0],%[fp1]\n"
		"\tfsne %[fail]\n"
	: [fp0]"=f"(fp0.d), [fp1]"=f"(fp1.d), [fail]"=dm"(fail)
	: [pi]"m"(pi)
	: "cc", "memory");
	CHECK(fail);
	EXPECT_FP(fp0, 0x40000000L, 0xc90fdaa2L, 0x2168C235L);
	EXPECT_FP(fp1, 0x40000000L, 0xc90fdaa2L, 0x2168C235L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x %[pi],%[fp0]\n"
		"\tfmove.b %[fp0],%[data]\n"
	: [fp0]"=f"(fp0.d)
	: [pi]"m"(pi), [data]"m"(data)
	: "cc", "memory");
	EXPECT((long)data.c[0], 3L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x %[pi],%[fp0]\n"
		"\tfmove.w %[fp0],%[data]\n"
	: [fp0]"=f"(fp0.d)
	: [pi]"m"(pi), [data]"m"(data)
	: "cc", "memory");
	EXPECT((long)data.w[0], 3L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x %[pi],%[fp0]\n"
		"\tfmove.l %[fp0],%[data]\n"
	: [fp0]"=f"(fp0.d)
	: [pi]"m"(pi), [data]"m"(data)
	: "cc", "memory");
	EXPECT((long)data.l[0], 3L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x %[pi],%[fp0]\n"
		"\tfmove.s %[fp0],%[data]\n"
	: [fp0]"=f"(fp0.d)
	: [pi]"m"(pi), [data]"m"(data)
	: "cc", "memory");
	EXPECT(data.l[0], 0x40490fdbL);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x %[pi],%[fp0]\n"
		"\tfmove.d %[fp0],%[data]\n"
	: [fp0]"=f"(fp0.d)
	: [pi]"m"(pi), [data]"m"(data)
	: "cc", "memory");
	EXPECT(data.l[0], 0x400921fbL);
	EXPECT(data.l[1], 0x54442d18L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x %[pi],%[fp0]\n"
		"\tfmove.x %[fp0],%[data]\n"
	: [fp0]"=f"(fp0.d)
	: [pi]"m"(pi), [data]"m"(data)
	: "cc", "memory");
	EXPECT(data.l[0], 0x40000000L);
	EXPECT(data.l[1], 0xc90fdaa2L);
	EXPECT(data.l[2], 0x2168C235L);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #-1,%[d3]\n"
		"\tfmove.b %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsuge %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #-1,%[d3]\n"
		"\tfmove.w %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsuge %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #-1,%[d3]\n"
		"\tfmove.l %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsuge %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #-1,%[d3]\n"
		"\tfmove.s %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsoge %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #1,%[d3]\n"
		"\tfmove.b %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsult %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #1,%[d3]\n"
		"\tfmove.w %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsult %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #1,%[d3]\n"
		"\tfmove.l %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsult %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #1,%[d3]\n"
		"\tfmove.s %[d3],%[fp0]\n"
		"\tftst.x %[fp0]\n"
		"\tfsult %[fail]\n"
	: [fp0]"=f"(fp0.d), [d3]"=d"(d3), [fail]"=dm"(fail)
	:
	: "cc", "memory");
	CHECK(fail);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.b #0xff,%[fp0]\n"
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0xbfff0000L, 0x80000000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.w #0xFABC,%[fp0]\n"
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0xc0090000L, 0xa8800000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.l #0xFABCDEFA,%[fp0]\n"
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0xc0190000L, 0xa86420c0L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.s #0xDEADBEAF,%[fp0]\n"
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0xc03d0000L, 0xadbeaf00L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.d #0x3ABCDEFADEADBEAF,%[fp0]\n"
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x3fab0000L, 0xe6f7d6f5L, 0x6df57800L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.x #0xc7000000DEADBEAF12345678,%[fp0]\n"
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0xc7000000L, 0xdeadbeafL, 0x12345678L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmove.p #0x088800012345678912345678,%[fp0]\n"
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x4b850000L, 0x909f850eL, 0x1505981aL);
	
	return status;
}
