/*
 * Test:USES_FPU
 */
#include "testdriver.h"

#ifndef __HAVE_68881__
 #error "must be compiled with FPU support"
#endif

int numtests;


int main(void)
{
	int status;
	union ld_union fp0;
	
	status = 0;
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x0,%[fp0]\n" /* Pi */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40000000L, 0xc90fdaa2L, 0x2168c235L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x0b,%[fp0]\n" /* log10(2) */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x3ffd0000L, 0x9a209a84L, 0xfbcff798L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x0c,%[fp0]\n" /* E */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40000000L, 0xadf85458L, 0xa2bb4a9aL);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x0d,%[fp0]\n" /* log2(e) */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x3fff0000L, 0xb8aa3b29L, 0x5c17f0bcL);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x0e,%[fp0]\n" /* log10(e) */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x3ffd0000L, 0xde5bd8a9L, 0x37287195L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x0f,%[fp0]\n" /* +0 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x00000000L, 0x00000000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x30,%[fp0]\n" /* ln(2) */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x3ffe0000L, 0xb17217f7L, 0xd1cf79acL);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x31,%[fp0]\n" /* ln(10) */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40000000L, 0x935d8dddL, 0xaaa8ac17L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x32,%[fp0]\n" /* 1e0 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x3fff0000L, 0x80000000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x33,%[fp0]\n" /* 1e1 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40020000L, 0xa0000000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x34,%[fp0]\n" /* 1e2 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40050000L, 0xc8000000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x35,%[fp0]\n" /* 1e4 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x400c0000L, 0x9c400000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x36,%[fp0]\n" /* 1e8 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40190000L, 0xbebc2000L, 0x00000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x37,%[fp0]\n" /* 1e16 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40340000L, 0x8e1bc9bfL, 0x04000000L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x38,%[fp0]\n" /* 1e32 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40690000L, 0x9dc5ada8L, 0x2b70b59eL)
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x39,%[fp0]\n" /* 1e64 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x40d30000L, 0xc2781f49L, 0xffcfa6d5L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x3A,%[fp0]\n" /* 1e128 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x41a80000L, 0x93ba47c9L, 0x80e98ce0L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x3B,%[fp0]\n" /* 1e256 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x43510000L, 0xaa7eebfbL, 0x9df9de8eL);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x3C,%[fp0]\n" /* 1e512 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x46a30000L, 0xe319a0aeL, 0xa60e91c7L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x3D,%[fp0]\n" /* 1e1024 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x4d480000L, 0xc9767586L, 0x81750c17L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x3E,%[fp0]\n" /* 1e2048 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x5a920000L, 0x9e8b3b5dL, 0xc53d5de5L);
	
	++numtests;
	__asm__ __volatile__(
		"\tfmovecr #0x3F,%[fp0]\n" /* 1e4096 */
	: [fp0]"=f"(fp0.d)
	:
	: "cc", "memory");
	EXPECT_FP(fp0, 0x75250000L, 0xc4605202L, 0x8a20979bL);
	
	return status;
}
