#include "testdriver.h"

int numtests;

int main(void)
{
	char fail, fail2;
	int status;
	long d1, d2, d3, result;
	
	status = 0;
	
	/* test that lsl is modulo 64 */
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #33,%[d1]\n"
		"\tmove.l #1,%[result]\n"
		"\tlsl.l %[d1],%[result]\n"
	: [result]"=d"(result), [d1]"=d"(d1)
	:
	: "cc", "memory");
	EXPECT(result, 0L);
	
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #65,%[d1]\n"
		"\tmove.l #1,%[result]\n"
		"\tlsl.l %[d1],%[result]\n"
	: [result]"=d"(result), [d1]"=d"(d1)
	:
	: "cc", "memory");
	EXPECT(result, 2L);

	/* test that zero shift does not affect X flag */
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #0,%[d1]\n"
		"\tmove.l #1,%[d2]\n"
		"\tmove.l #1,%[d3]\n"
		"\tmove.l #0,%[result]\n"
		"\tlsr.l %[d2],%[d3]\n" /* should set X & C */
		"\tscc %[fail]\n"
		"\tlsr.l %[d1],%[result]\n" /* should not affect X & clear C */
		"\tscs %[fail2]\n"
		"\taddx.l %[result],%[result]\n"
	: [result]"=d"(result), [fail]"=dm"(fail), [fail2]"=dm"(fail2), [d1]"=d"(d1), [d2]"=d"(d2), [d3]"=d"(d3)
	:
	: "cc", "memory");
	CHECK(fail);
	CHECK(fail2);
	EXPECT(result, 1L);
	
	/* test that both C & X flags are set to the last bit shifted out */
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #0x55555555,%[d1]\n"
		"\tmove.l #0,%[d2]\n"
		"\tlsl.l #8,%[d1]\n"
		"\tscc %[fail]\n"
		"\taddx.l %[d2],%[d2]\n"
		"\tmove.l #0,%[result]\n"
		"\tlsl.l #1,%[d1]\n"
		"\tscs %[fail2]\n"
		"\taddx.l %[result],%[result]\n"
	: [result]"=d"(result), [fail]"=dm"(fail), [fail2]"=dm"(fail2), [d1]"=d"(d1), [d2]"=d"(d2)
	:
	: "cc", "memory");
	CHECK(fail);
	CHECK(fail2);
	EXPECT(result, 0L);
	EXPECT(d1, 0xaaaaaa00L);
	EXPECT(d2, 1L);
	
	/* same as above, for word shifts */
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #0x55555555,%[d1]\n"
		"\tmove.l #0,%[d2]\n"
		"\tlsl.w #8,%[d1]\n"
		"\tscc %[fail]\n"
		"\taddx.l %[d2],%[d2]\n"
		"\tmove.l #0,%[result]\n"
		"\tlsl.w #1,%[d1]\n"
		"\tscs %[fail2]\n"
		"\taddx.l %[result],%[result]\n"
	: [result]"=d"(result), [fail]"=dm"(fail), [fail2]"=dm"(fail2), [d1]"=d"(d1), [d2]"=d"(d2)
	:
	: "cc", "memory");
	CHECK(fail);
	CHECK(fail2);
	EXPECT(result, 0L);
	EXPECT(d1, 0x5555aa00L);
	EXPECT(d2, 1L);
	
	/* same as above, for byte shifts */
	++numtests;
	__asm__ __volatile__(
		"\tmove.l #0x55555555,%[d1]\n"
		"\tmove.l #0,%[d2]\n"
		"\tlsl.b #8,%[d1]\n"
		"\tscc %[fail]\n"
		"\taddx.l %[d2],%[d2]\n"
		"\tmove.l #0,%[result]\n"
		"\tlsl.b #1,%[d1]\n"
		"\tscs %[fail2]\n"
		"\taddx.l %[result],%[result]\n"
	: [result]"=d"(result), [fail]"=dm"(fail), [fail2]"=dm"(fail2), [d1]"=d"(d1), [d2]"=d"(d2)
	:
	: "cc", "memory");
	CHECK(fail);
	CHECK(fail2);
	EXPECT(result, 0L);
	EXPECT(d1, 0x55555500L);
	EXPECT(d2, 1L);
	
	return status;
}
